using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Xml;
using System.Web;
using System.Web.Security;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;

using GBPVR.Public;

namespace gbweb
{
	/// <summary>
	/// Summary description for Login.
	/// </summary>
	public class Login : System.Web.UI.Page
	{
		protected System.Web.UI.WebControls.Label invalidCredentialsLabel;
		protected System.Web.UI.HtmlControls.HtmlForm Form1;
		protected System.Web.UI.WebControls.LinkButton LoginButton;
		protected System.Web.UI.HtmlControls.HtmlInputText Username;
		protected System.Web.UI.HtmlControls.HtmlInputText Password;
		protected System.Web.UI.HtmlControls.HtmlInputHidden PasswordHash;
		protected string salt;
		private String requiredUsername;
		protected System.Web.UI.HtmlControls.HtmlImage logo;
		private String requiredPasswordHash;
	
		private void Page_Load(object sender, System.EventArgs e)
		{
			logo.Src = logo.Src.Replace("images", Global.ThemePath);
			
			XmlDocument configDoc = Global.Config;

			// extract username and password
			requiredUsername = "admin";
			requiredPasswordHash = string.Empty;
			XmlNode node = configDoc.SelectSingleNode("/settings/WebUsername");
			if (node != null)
			{
				requiredUsername = node.InnerText;
			}
			node = configDoc.SelectSingleNode("/settings/WebPassword");
			if (node != null)
			{
				requiredPasswordHash = node.InnerText;
			}

			if (!IsPostBack && (Global.Settings.autologinNets.CheckNumber(Request.UserHostAddress)))
			{
				CompleteLogin();
			}

			logo.Attributes.Add("title", Global.Settings.GetVersionToolTip());

			// allocate salt if we havn't already
			if (Session["LoginSalt"] == null)
			{
				Session["LoginSalt"] = System.Guid.NewGuid().ToString();
			}
			salt = (string)Session["LoginSalt"];

			Page.RegisterHiddenField("__EVENTTARGET", LoginButton.UniqueID);

		}

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			this.LoginButton.Click += new System.EventHandler(this.LoginButton_Click);
			this.ID = "LoginButton";
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion

		private void LoginButton_Click(object sender, System.EventArgs e)
		{
			// check credentials


			// generate the expected hash
			string expectedHash = FormsAuthentication.HashPasswordForStoringInConfigFile(requiredPasswordHash.ToLower() + salt, "MD5");


			// direct user to "manage listings" if credentials are correct
			invalidCredentialsLabel.Visible = true;				
			if ((Username.Value == requiredUsername) &&
				(PasswordHash.Value == expectedHash.ToLower()))
			{
				CompleteLogin();
			}
			else
			{
				Logger.Warning("Failed login for " + Username.Value + " from " + Request.UserHostAddress);
			}
		}

		private void CompleteLogin()
		{
			string userName = Username.Value;
			bool createPersistentCookie = false;
			string cookiePath = FormsAuthentication.FormsCookiePath;
			string redirectUrl = FormsAuthentication.GetRedirectUrl(userName, createPersistentCookie);
			Uri redirectUri = new Uri(Request.Url, redirectUrl);

			if ((Username.Value.Length == 0) && (
				(string.Compare(redirectUri.LocalPath, Request.ApplicationPath + "/Admin.aspx", true) == 0)))
			{
				return;
			}

			if (	
				(string.Compare(redirectUri.LocalPath, Request.ApplicationPath + "/Default.aspx", true) == 0) || 
				(string.Compare(redirectUri.LocalPath, Request.ApplicationPath + "/Logout.aspx", true) == 0))
			{
				redirectUrl = Request.ApplicationPath + "/Guide.aspx";
			}

			if (Username.Value.Length == 0)
			{
				Logger.Info("Automatic login from " + Request.UserHostAddress);
			}
			else
			{
				Logger.Info("Successful login for " + Username.Value + " from " + Request.UserHostAddress);
			}

			FormsAuthentication.Initialize();
			FormsAuthentication.SetAuthCookie(Username.Value, createPersistentCookie, cookiePath);
			Response.Redirect(redirectUrl, false);
		}
	}
}
