using System;
using System.Collections;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

using GBPVR.Public;

using gbweb.classes;

namespace gbweb
{
	/// <summary>
	/// Summary description for Manage.
	/// </summary>
	public class Manage : System.Web.UI.Page
	{
		protected System.Web.UI.WebControls.CheckBoxList recordingTypes;
		protected System.Web.UI.WebControls.Table schedView;

		Hashtable channelCache = new Hashtable();

		private void checkRecordingType(ref int recordingTypeFilter, string text, int value)
		{
			ListItem item = recordingTypes.Items.FindByValue(value.ToString());
			int flag = (int)Math.Pow(2, value);
			if (item == null)
			{
				item = new ListItem(text, value.ToString());
				recordingTypes.Items.Add(item);
				item.Selected = (recordingTypeFilter & flag) != 0;
			}
			else
			{
				if (item.Selected)
					recordingTypeFilter = recordingTypeFilter | flag;
				else
					recordingTypeFilter = recordingTypeFilter & (0xffff - flag);
			}
		}

		private void Page_Load(object sender, System.EventArgs e)
		{
			HttpCookie cookie = Request.Cookies["recordingTypeFilter"];
			int recordingTypeFilter = cookie != null ? Convert.ToInt32(cookie.Value) : 0xffbf;
			checkRecordingType(ref recordingTypeFilter, "Pending", ScheduledRecording.STATUS_PENDING);
			checkRecordingType(ref recordingTypeFilter, "In Progress", ScheduledRecording.STATUS_IN_PROGRESS);
			checkRecordingType(ref recordingTypeFilter, "Available", ScheduledRecording.STATUS_COMPLETED);
			checkRecordingType(ref recordingTypeFilter, "Failed", ScheduledRecording.STATUS_COMPLETED_WITH_ERROR);
			checkRecordingType(ref recordingTypeFilter, "Conflict", ScheduledRecording.STATUS_CONFLICT);
			checkRecordingType(ref recordingTypeFilter, "Reoccurring", ScheduledRecording.STATUS_PLACE_HOLDER);
			checkRecordingType(ref recordingTypeFilter, "Deleted", ScheduledRecording.STATUS_DELETED);
			cookie = new HttpCookie("recordingTypeFilter", recordingTypeFilter.ToString());
			cookie.Expires = DateTime.Now.AddYears(1);
			Response.Cookies.Add(cookie);

			cookie = Request.Cookies["sortOrder"];
			string sortOrder = cookie != null ? cookie.Value.Trim(',') : "datetime,channel,title,status";
			ArrayList sortOrders = new ArrayList(sortOrder.Split(','));
			string newSort = Request["sort"];
			if (newSort != null)
			{
				if (sortOrders.Contains(newSort))
				{
					if (sortOrders[0].ToString() == newSort)
					{
						sortOrders.Remove(newSort);
						sortOrders.Insert(0, newSort + " desc");
					}
					else
					{
						sortOrders.Remove(newSort);
						sortOrders.Insert(0, newSort);
					}
				}
				else
				{
					sortOrders.Remove(newSort + " desc");
					sortOrders.Insert(0, newSort);
				}
			}
			sortOrder = string.Join(",", (string[])sortOrders.ToArray(typeof(string)));
			cookie = new HttpCookie("sortOrder", sortOrder);
			cookie.Expires = DateTime.Now.AddYears(1);
			Response.Cookies.Add(cookie);

			Schedule scheduleHelper = Global.Schedule;
			ProgrammeDisplay displayProgramme = new ProgrammeDisplay();

			// build up a list of the recording we already know about
			IDictionary knownRecordings = scheduleHelper.LoadKnownRecordings();

			string cancel = Request.Params["cancel"];
			if ((cancel != null) && (cancel.Length > 0))
			{
				ScheduledRecording scheduledRecording = scheduleHelper.GetScheduledRecordingByOID(Convert.ToInt32(cancel));
				if (scheduledRecording != null)
				{
					scheduleHelper.CancelScheduledRecording(scheduledRecording);
				}
			}

			IList myScheduledRecordings = scheduleHelper.LoadRecordingSchedule();
			foreach (ScheduledRecording scheduledRecording in myScheduledRecordings)
			{
				int oid = scheduledRecording.getChannelOID();
				if (!channelCache.Contains(oid))
				{
					Channel channel = scheduleHelper.GetChannelByOID(oid);
					channelCache.Add(oid, channel);
				}
			}

			// put table header
			TableRow headerRow = new TableRow();

			TableCell colStatus = new TableCell();
			colStatus.Text = "<a href=\"?sort=status\"><b>Status</b></a>" + getSortDescription(sortOrders[0], "status");
			colStatus.CssClass = "header";
			colStatus.Width = 150;
			colStatus.Wrap = false;
			headerRow.Cells.Add(colStatus);

			TableCell colAirDate = new TableCell();
			colAirDate.Text = "<a href=\"?sort=datetime\">Air Date</a>" + getSortDescription(sortOrders[0], "datetime");
			colAirDate.CssClass = "header";
			colAirDate.Wrap = false;
			headerRow.Cells.Add(colAirDate);

			TableCell colChannel = new TableCell();
			colChannel.Text = "<a href=\"?sort=channel\">Channel</a>" + getSortDescription(sortOrders[0], "channel");
			colChannel.CssClass = "header";
			colChannel.Wrap = false;
			headerRow.Cells.Add(colChannel);

			TableCell colShow = new TableCell();
			colShow.Text = "<a href=\"?sort=title\">Show</a>" + getSortDescription(sortOrders[0], "title");
			colShow.CssClass = "header";
			colShow.Wrap = false;
			headerRow.Cells.Add(colShow);

			schedView.Rows.Add(headerRow);

			if (myScheduledRecordings is ArrayList)
				((ArrayList)myScheduledRecordings).Sort(new RecordingComparer(sortOrders, channelCache));

			TableRow recordingRow;
			foreach (ScheduledRecording scheduledRecording in myScheduledRecordings)
			{
				int flag = (int)Math.Pow(2, scheduledRecording.getRecordingStatus());
				if ((recordingTypeFilter & flag) == 0) continue;

				recordingRow = new TableRow();
				recordingRow.CssClass = "tbg";

				colStatus = new TableCell();
				colStatus.VerticalAlign = VerticalAlign.Top;

				colAirDate = new TableCell();
				colAirDate.VerticalAlign = VerticalAlign.Top;
				colAirDate.Wrap = false;

				colChannel = new TableCell();
				colChannel.VerticalAlign = VerticalAlign.Top;

				colShow = new TableCell();
				colShow.VerticalAlign = VerticalAlign.Top;
				String quality;

				switch (scheduledRecording.getQualitySetting())
				{
					case (int)Schedule.Quality.Low:
						quality = "Low";
						break;
					case (int)Schedule.Quality.Medium:
						quality = "Medium";
						break;
					case (int)Schedule.Quality.High:
						quality = "High";
						break;
					case (int)Schedule.Quality.Custom1:
						quality = "Custom 1";
						break;
					case (int)Schedule.Quality.Custom2:
						quality = "Custom 2";
						break;
					case (int)Schedule.Quality.LiveTV:
						quality = "LiveTV";
						break;
					default:
						quality = string.Empty;
						break;
				}

				Programme programme = scheduledRecording.getProgramme();

				switch (scheduledRecording.getRecordingStatus())
				{
					case ScheduledRecording.STATUS_PENDING:
						colStatus.CssClass = "pending";
						colStatus.Text = "Pending<br/>" +
							"<img src=\"public/t.gif\" width=\"1\" height=\"4\"/><br/>" +
							"Quality: <i>" + quality + "</i><br>" +
							"<a href=\"Manage.aspx?cancel=" + scheduledRecording.getOID() + "\" class=\"btn-normal-small-red\"><span>Cancel Recording</span></a>";
						break;
					case ScheduledRecording.STATUS_IN_PROGRESS:
						colStatus.CssClass = "inprogress";
						colStatus.Text = "In Progress<br/>" +
							"<img src=\"public/t.gif\" width=\"1\" height=\"4\"/><br/>" +
							"Quality: <i>" + quality + "</i><br>" +
							"<a href=\"" + Download.GetDownloadUrl(false, false, scheduledRecording.getOID()) + "\" class=\"btn-normal-small\"><span>Play Program</span></a>" +
							"<a href=\"" + Download.GetDownloadUrl(true, true, scheduledRecording.getOID()) + "\" class=\"btn-normal-small\"><span>Download</span></a>";
						break;
					case ScheduledRecording.STATUS_COMPLETED:
						colStatus.CssClass = "available";
						colStatus.Text = "Available<br/>" +
							"<img src=\"public/t.gif\" width=\"1\" height=\"4\"/><br/>" +
							"Quality: <i>" + quality + "</i><br>" +
							"<a href=\"" + Download.GetDownloadUrl(false, false, scheduledRecording.getOID()) + "\" class=\"btn-normal-small\"><span>Play Program</span></a>" +
							"<a href=\"" + Download.GetDownloadUrl(true, true, scheduledRecording.getOID()) + "\" class=\"btn-normal-small\"><span>Download</span></a>";
						break;
					case ScheduledRecording.STATUS_COMPLETED_WITH_ERROR:
						colStatus.CssClass = "failed";
						colStatus.Text = "Failed<br/>" +
							"<img src=\"public/t.gif\" width=\"1\" height=\"4\"/><br/>" +
							"Quality: <i>" + quality + "</i><br>" +
							"<a href=\"Manage.aspx?cancel=" + scheduledRecording.getOID() + "\" class=\"btn-normal-small\"><span>Delete</span></a>";
						break;
					case ScheduledRecording.STATUS_CONFLICT:
						colStatus.CssClass = "conflict";
						colStatus.Text = "Conflict<br/>" +
							"<img src=\"public/t.gif\" width=\"1\" height=\"4\"/><br/>" +
							"Quality: <i>" + quality + "</i><br>" +
							"<a href=\"Manage.aspx?cancel=" + scheduledRecording.getOID() + "\" class=\"btn-normal-small\"><span>Delete</span></a>";
						break;
					case ScheduledRecording.STATUS_PLACE_HOLDER:
						colStatus.CssClass = "reoccurring";
						colStatus.Text = "Reoccurring<br/>" +
							"<img src=\"public/t.gif\" width=\"1\" height=\"4\"/><br/>" +
							"Quality: <i>" + quality + "</i><br>" +
							"<a href=\"Manage.aspx?cancel=" + scheduledRecording.getOID() + "\" class=\"btn-normal-small-red\"><span>Cancel Recording</span></a>";
						break;
					case ScheduledRecording.STATUS_DELETED:
						colStatus.CssClass = "deleted";
						colStatus.Text = "Deleted<br/>";
						break;
					default:
						colStatus.Text = "NONE!!";
						break;
				}

				// show air date
				string date = null;
				string time = null;
				switch (scheduledRecording.getRecordingType())
				{
					case ScheduledRecording.TYPE_RECORD_ONCE:
						date = scheduledRecording.getStartTime().ToLongDateString();
						time = scheduledRecording.getStartTime().ToShortTimeString() + " - " + scheduledRecording.getEndTime().ToShortTimeString();
						break;
					case ScheduledRecording.TYPE_RECORD_DAILY:
						date = "Daily";
						time = scheduledRecording.getStartTime().ToShortTimeString() + " - " + scheduledRecording.getEndTime().ToShortTimeString();
						break;
					case ScheduledRecording.TYPE_RECORD_WEEK_DAYS:
						date = "Week Days";
						time = scheduledRecording.getStartTime().ToShortTimeString() + " - " + scheduledRecording.getEndTime().ToShortTimeString();
						break;
					case ScheduledRecording.TYPE_RECORD_WEEKEND_DAYS:
						date = "Weekend Days";
						time = scheduledRecording.getStartTime().ToShortTimeString() + " - " + scheduledRecording.getEndTime().ToShortTimeString();
						break;
					case ScheduledRecording.TYPE_RECORD_WEEKLY:
						date = "Weekly: " + scheduledRecording.getStartTime().DayOfWeek.ToString();
						time = scheduledRecording.getStartTime().ToShortTimeString() + " - " + scheduledRecording.getEndTime().ToShortTimeString();
						break;
					case ScheduledRecording.TYPE_RECORD_SEASON:
						date = "Season";
						if (scheduledRecording.getStartTime().Year != 2001)
							time = scheduledRecording.getStartTime().ToShortTimeString() + " - " + scheduledRecording.getEndTime().ToShortTimeString();
						break;
				}
				colAirDate.Text = "" + date + "<br>" + time;

				// show channel
				Channel channel = (Channel)channelCache[scheduledRecording.getChannelOID()];
				if (channel != null)
				{
					string channelName = channel.getName();
					if (channelName.StartsWith(channel.getChannelNumber().ToString())) // cosmetic stuff for my american friends
					{
						channelName = channelName.Substring(channel.getChannelNumber().ToString().Length);
					}
					colChannel.Text = "<span class=\"channelnumber\"><b>" + channel.getChannelNumber() + "</b></span><br><span class=\"channelname\"><nobr>" + channelName + "</nobr></span>";
				}

				displayProgramme.FillProgrammeDisplay(Server, colShow, programme, scheduledRecording, false);

				colStatus.CssClass = colShow.CssClass + " " + colStatus.CssClass;
				colAirDate.CssClass = colShow.CssClass;
				colChannel.CssClass = colShow.CssClass;

				recordingRow.Cells.Add(colStatus);
				recordingRow.Cells.Add(colAirDate);
				recordingRow.Cells.Add(colChannel);
				recordingRow.Cells.Add(colShow);

				schedView.Rows.Add(recordingRow);

			}
			displayProgramme.Dispose();

		}

		string getSortDescription(object selectedSort, string thisSort)
		{
			if (selectedSort.ToString() == thisSort)
				return " (asc)";
			else if (selectedSort.ToString() == thisSort + " desc")
				return " (desc)";
			else
				return string.Empty;
		}

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}

		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion

		class RecordingComparer : IComparer
		{

			IList sortKeys;
			Hashtable channelCache;

			public RecordingComparer(IList SortKeys, Hashtable ChannelCache)
			{
				sortKeys = SortKeys;
				channelCache = ChannelCache;
			}

			IComparable getRecordingValue(object obj, string key)
			{
				ScheduledRecording scheduledRecording = (ScheduledRecording)obj;
				switch (key)
				{
					case "status":
					{
						switch (scheduledRecording.getRecordingStatus())
						{
							case ScheduledRecording.STATUS_PENDING:
								return "Pending";
							case ScheduledRecording.STATUS_IN_PROGRESS:
								return "In Progress";
							case ScheduledRecording.STATUS_COMPLETED:
								return "Available";
							case ScheduledRecording.STATUS_COMPLETED_WITH_ERROR:
								return "Failed";
							case ScheduledRecording.STATUS_CONFLICT:
								return "Conflict";
							case ScheduledRecording.STATUS_PLACE_HOLDER:
								return "Reoccurring";
							case ScheduledRecording.STATUS_DELETED:
								return "Deleted";
							default:
								return "NONE!!";
						}
					}
					case "datetime":
					{
						switch (scheduledRecording.getRecordingType())
						{
							case ScheduledRecording.TYPE_RECORD_ONCE:
								return scheduledRecording.getStartTime().Ticks;
							default:
								return scheduledRecording.getStartTime().TimeOfDay.Ticks;
						}
					}
					case "channel":
					{
						Channel channel = (Channel)channelCache[scheduledRecording.getChannelOID()];
						if (channel != null)
						{
							return channel.getChannelNumber();
						}
						return 0;
					}
					case "title":
					{
						Programme programme = scheduledRecording.getProgramme();
						switch (scheduledRecording.getRecordingType())
						{
							case ScheduledRecording.TYPE_RECORD_ONCE:
								if (programme == null)
								{
									return "Manual Recording";
								}
								else
								{
									if (programme.getSubTitle().Length > 0)
									{
										return programme.getTitle() + ": " + programme.getSubTitle();
									}
									else
									{
										return programme.getTitle();
									}
								}
							default:
								return scheduledRecording.getFileName();
						}
					}
					default:
						return string.Empty;
				}
			}

			#region IComparer Members

			public int Compare(object x, object y)
			{
				for (int i = 0; i < sortKeys.Count; i++)
				{
					string key = sortKeys[i].ToString();
					int order = 1;
					if (key.EndsWith(" desc"))
					{
						key = key.Substring(0, key.Length - 5);
						order = -1;
					}
					int result = getRecordingValue(x, key).CompareTo(getRecordingValue(y, key));
					if (result != 0) return result * order;
				}
				return 0;
			}

			#endregion

		}
	}
}
