using System;
using System.Collections;
using System.Collections.Specialized;
using System.IO;
using System.Reflection;
using System.Xml;
using System.Web;

using Microsoft.Win32;

using GBPVR.Public;

using IPNumbers;

namespace gbweb
{
	/// <summary>
	/// Summary description for Settings.
	/// </summary>
	public class Settings
	{
		public string schedule = "GBPVR";
		public int minuteSpan = 120;
		public int minuteInterval = 30;
		public int minuteSlice = 1;
		public int timelineRepeat = 10;
		public string theme = "default";
		public string recordingQuality = "Medium";
		public bool showSubtitle = true;
		public bool showDescription = true;
		public string showGenre = "singleGenre";
		public bool showRating = true;
		public bool showRepeat = true;
		public bool showIMDB = true;
		public bool showNetflix = true;
		public bool showCredits = true;
		public bool useChannelIcons = false;
		public string channelIconHeight = "";
		public string channelIconWidth = "";
		public string channelIconExtensions = "jpg,png,gif";
		public string playlistExtension = "m3u";
		public string musicFiles = @"^.*\.(mp3|wav|ogg|wma|ape)$";
		public string videoFiles = @"^.*\.(mpeg|mpg|m2v|avi|ogm|mp4|wmv|iso|rm|dvr-ms)$";
		public string coverFiles = @"^(cover|folder|albumart)\.(jpg|gif|png|bmp)$";
		public string autologinNetworks = "127.0.0.0/8,10.0.0.0/8,172.16.0.0/12,192.168.0.0/16,169.254.0.0/16";
		
		[NonSerialized()]
		public IPList autologinNets;

		public Settings()
		{
			//
			// TODO: Add constructor logic here
			//
			Load();
		}

		private void Load()
		{
			string value;
			value = GetSetting(this, "schedule");			if ((value != null) && (value.Length > 0)) schedule = value;
			value = GetSetting(this, "theme");				if ((value != null) && (value.Length > 0)) theme = value;
			value = GetSetting(this, "minuteSpan");			if ((value != null) && (value.Length > 0)) minuteSpan = Convert.ToInt32(value);
			value = GetSetting(this, "minuteInterval");		if ((value != null) && (value.Length > 0)) minuteInterval = Convert.ToInt32(value);
			value = GetSetting(this, "minuteSlice");		if ((value != null) && (value.Length > 0)) minuteSlice = Convert.ToInt32(value);
			value = GetSetting(this, "timelineRepeat");		if ((value != null) && (value.Length > 0)) timelineRepeat = Convert.ToInt32(value);
			value = GetSetting(this, "recordingQuality");	if ((value != null) && (value.Length > 0)) recordingQuality = value;
			value = GetSetting(this, "showDescription");	if ((value != null) && (value.Length > 0)) showDescription = Convert.ToBoolean(value);
			value = GetSetting(this, "showSubtitle");		if ((value != null) && (value.Length > 0)) showSubtitle = Convert.ToBoolean(value);
			value = GetSetting(this, "showGenre");			if ((value != null) && (value.Length > 0)) showGenre = value;
			if (showGenre.ToLower() == "true")				showGenre = "singleGenre";
			if (showGenre.ToLower() == "false")				showGenre = "noGenre";
			value = GetSetting(this, "showRating");			if ((value != null) && (value.Length > 0)) showRating = Convert.ToBoolean(value);
			value = GetSetting(this, "showRepeat");			if ((value != null) && (value.Length > 0)) showRepeat = Convert.ToBoolean(value);
			value = GetSetting(this, "showIMDB");			if ((value != null) && (value.Length > 0)) showIMDB = Convert.ToBoolean(value);
			value = GetSetting(this, "showNetflix");		if ((value != null) && (value.Length > 0)) showNetflix = Convert.ToBoolean(value);
			value = GetSetting(this, "showCredits");		if ((value != null) && (value.Length > 0)) showCredits = Convert.ToBoolean(value);
			value = GetSetting(this, "UseChannelIcons");	if ((value != null) && (value.Length > 0)) useChannelIcons = Convert.ToBoolean(value);
			value = GetSetting(this, "playlistExtnesion");	if ((value != null) && (value.Length > 0)) playlistExtension = value;
			value = GetSetting(this, "chnlIconH");			if ((value != null) && (value.Length > 0)) channelIconHeight = value;
			value = GetSetting(this, "chnlIconW");			if ((value != null) && (value.Length > 0)) channelIconWidth = value;
			value = GetSetting(this, "autologinNetworks");	if ((value != null) && (value.Length > 0)) autologinNetworks = value;

			string[] invalidAutologinNetworks = ParseAutologinNetworks();
			if (invalidAutologinNetworks.Length > 0)
			{
				Logger.Warning("Invalid autologinNetworks: " + string.Join(",", invalidAutologinNetworks));
			}
		}

		public void Save()
		{
			SetSetting(this, "schedule"				, schedule);
			SetSetting(this, "theme"				, theme);
			SetSetting(this, "minuteSpan"			, minuteSpan.ToString());
			SetSetting(this, "minuteInterval"		, minuteInterval.ToString());
			SetSetting(this, "minuteSlice"			, minuteSlice.ToString());
			SetSetting(this, "timelineRepeat"		, timelineRepeat.ToString());
			SetSetting(this, "recordingQuality"		, recordingQuality);
			SetSetting(this, "showDescription"		, showDescription.ToString());
			SetSetting(this, "showSubtitle"			, showSubtitle.ToString());
			SetSetting(this, "showGenre"			, showGenre);
			SetSetting(this, "showRating"			, showRating.ToString());
			SetSetting(this, "showRepeat"			, showRepeat.ToString());
			SetSetting(this, "showIMDB"				, showIMDB.ToString());
			SetSetting(this, "showNetflix"			, showNetflix.ToString());
			SetSetting(this, "showCredits"			, showCredits.ToString());
			SetSetting(this, "UseChannelIcons"		, useChannelIcons.ToString());
			SetSetting(this, "playlistExtnesion"	, playlistExtension);
			SetSetting(this, "chnlIconH"			, channelIconHeight);
			SetSetting(this, "chnlIconW"			, channelIconWidth);
			SetSetting(this, "autologinNetworks"	, autologinNetworks);
			SaveToFile();
		}

		public string[] ParseAutologinNetworks()
		{
			ArrayList invalidAutologinNetworks = new ArrayList();
			autologinNets = new IPList();
			if (autologinNetworks.Length > 0)
			{
				foreach (string autologinNetwork in autologinNetworks.Split(','))
				{
					if (autologinNetwork.ToLower() == "none") continue;
					try 
					{
						string[] networkDetails = autologinNetwork.Split('/');
						if (networkDetails.Length == 1)
						{
							autologinNets.Add(networkDetails[0]);
						}
						else if (networkDetails.Length == 2)
						{
							int parts = networkDetails[1].Split('.').Length;
							if (parts == 1)
							{
								autologinNets.Add(networkDetails[0], Convert.ToInt32(networkDetails[1]));
							}
							else if (parts == 4)
							{
								autologinNets.Add(networkDetails[0], networkDetails[1]);
							}
							else
							{
								invalidAutologinNetworks.Add(autologinNetwork);
							}
						}
						else
						{
							invalidAutologinNetworks.Add(autologinNetwork);
						}
					}
					catch
					{
						invalidAutologinNetworks.Add(autologinNetwork);
					}
				}
			}
			return (string[])invalidAutologinNetworks.ToArray(typeof(string));
		}

		private static string versionToolTip;
		public string GetVersionToolTip()
		{
			if (versionToolTip == null)
			{
				Assembly me = Assembly.GetExecutingAssembly();
				string configuration = ((AssemblyConfigurationAttribute)me.GetCustomAttributes(typeof(AssemblyConfigurationAttribute), false)[0]).Configuration;
				string version = me.GetName().Version.ToString(2);
				XmlDocument changeLog = new XmlDocument();
				changeLog.Load(HttpContext.Current.Server.MapPath("~/public/ChangeLog.xml"));
				string[] revision = changeLog.SelectSingleNode("/changelog/@commit_revision").Value.Split(new char[] {' ', '$'});
				versionToolTip = "GB-PVR web server, " + configuration + ", version: " + version + "." + revision[2];
			}
			return versionToolTip;
		}

		#region Private Variables

		private HybridDictionary globalSettings;
		private string webConfigFileName;
		private XmlDocument configWeb;

		#endregion

		#region Public Global Settings Methods

		public string GetSetting(object moduleObject, string settingName)
		{
			string moduleName = moduleObject.GetType().Assembly.GetName().Name;

			Logger.Verbose("RecorderHelperConfig: GetSetting(" + moduleName + ", " + settingName + ")");

			LoadFromFile(false);

			XmlNode globalSettingsNode = (XmlNode)globalSettings[moduleName];

			if (globalSettingsNode == null) return null;

			XmlNode settingNode = globalSettingsNode.SelectSingleNode(settingName);

			if (settingNode == null) return null;

			return settingNode.InnerText;
		}

		public void SetSetting(object moduleObject, string settingName, string value)
		{
			string moduleName = moduleObject.GetType().Assembly.GetName().Name;

			Logger.Verbose("RecorderHelperConfig: SetSetting(" + moduleName + ", " + settingName + ")");

			XmlNode globalSettingsNode = (XmlNode)globalSettings[moduleName];

			if (globalSettingsNode == null)
			{
				globalSettingsNode = configWeb.CreateElement(moduleName);
				globalSettings[moduleName] = globalSettingsNode;
			}

			XmlNode settingNode = globalSettingsNode.SelectSingleNode(settingName);

			if (settingNode == null)
			{
				settingNode = configWeb.CreateElement(settingName);
				globalSettingsNode.AppendChild(settingNode);
			}

			settingNode.InnerText = value;
		}

		public void SaveSettings()
		{
			SaveToFile();
		}

		#endregion

		#region Private Methods

		private void LoadFromFile(bool reload)
		{
			if (!reload && (configWeb != null)) return;

			Logger.Verbose("RecorderHelperConfig: LoadFromFile()");

			configWeb = new XmlDocument();

			if (File.Exists(GetWebConfigFileName()))
				configWeb.Load(GetWebConfigFileName());

			globalSettings = new HybridDictionary();

			foreach (XmlNode globalSettingsNode in configWeb.SelectNodes("settings/globalSettings/*"))
			{
				globalSettings[globalSettingsNode.Name] = globalSettingsNode;
			}

		}

		private void SaveToFile()
		{
			Logger.Verbose("RecorderHelperConfig: SaveToFile()");

			configWeb.RemoveAll();

			configWeb.AppendChild(configWeb.CreateXmlDeclaration("1.0", "UTF-8", null));

			XmlNode settings = configWeb.CreateElement("settings");
			configWeb.AppendChild(settings);

			XmlNode global = configWeb.CreateElement("globalSettings");
			settings.AppendChild(global);

			ArrayList globalSettingsKeys = new ArrayList(globalSettings.Keys);
			globalSettingsKeys.Sort();
			foreach (string globalSettingKey in globalSettingsKeys)
			{
				global.AppendChild((XmlNode)globalSettings[globalSettingKey]);
			}

			configWeb.Save(GetWebConfigFileName());
		}

		private string GetWebConfigFileName()
		{
			if (webConfigFileName == null)
			{
				webConfigFileName = Path.Combine(GetInstallDir(), "config-web.xml");
			}
			return webConfigFileName;
		}

		public string GetInstallDir()
		{
			// determine install location
			string installDirectory = @"c:\program files\devnz\gbpvr\";
			try
			{
				using (RegistryKey key = Registry.LocalMachine.OpenSubKey(@"SOFTWARE\devnz\"))
				{
					string temp = (string)key.GetValue("GBPVR InstallDir");
					if (temp != null)
					{
						installDirectory = temp;
					}
				}
			}
			catch (Exception ex)
			{
				Logger.Error("Error reading install location from registry: " + ex.ToString());
			}
			return installDirectory;
		}

		#endregion

	}
}
