using System;
using System.Collections;
using System.Data.OleDb;
using System.IO;
using System.Web;

namespace gbweb
{
	public class ZapTools
	{
		public enum Role
		{
			Actor = 1,
			Director = 2,
			Producer = 3,
			Writer = 4,
			ExecProducer = 5,
			GuestStar = 6,
			Host = 7
		}

		private static string databaseFileName;

		private static OleDbConnection zapConnection
		{
			get
			{
				return HttpContext.Current.Items["zapConnection"] as OleDbConnection;
			}
			set
			{
				HttpContext.Current.Items["zapConnection"] = value;
			}
		}

		// TODO: All these statics are creating a race condition on page processing
		public static bool Initialize()
		{
			if (databaseFileName == null)
			{
				databaseFileName = Path.Combine(Global.Settings.GetInstallDir(), "zapdb.mdb");
			}
			return File.Exists(databaseFileName);
		}

		public static bool checkZapEPG()
		{
			using (OleDbConnection checkZapConnection = Global.GetOpenGBPVRDbConnection())
			{
				// create the command object and store the sql query
				OleDbCommand aCommand = new OleDbCommand("select capture_source_oid from ZAP2IT_EPGSOURCE", checkZapConnection);
				OleDbDataReader aReader = aCommand.ExecuteReader();

				//If any rows were returned then the user uses the Zap2it EPG
				bool checkZap = false;
				if (aReader.HasRows)
				{
					checkZap = true;
				}

				//close the reader
				checkZapConnection.Close();
				return checkZap;
			}
		}

		public static void OpenZapDB()
		{
			zapConnection = new OleDbConnection("Provider=Microsoft.Jet.OLEDB.4.0;Data Source=" + databaseFileName);
			zapConnection.Open();
		}

		public static void CloseZapDB()
		{
			zapConnection.Close();
		}

		public static bool isRepeat(string UniqueProgrammeIdentifier)
		{
			byte isRepeatValue = 0;

			// create the command object and store the sql query
			OleDbCommand aCommand = new OleDbCommand("select repeat from Schedule where scheduleid='" + UniqueProgrammeIdentifier + "'", zapConnection);
			OleDbDataReader aReader = aCommand.ExecuteReader();

			//Iterate throuth the database
			while (aReader.Read())
			{
				isRepeatValue = aReader.GetByte(0);
			}

			//close the reader
			aReader.Close();

			return Convert.ToBoolean(isRepeatValue);
		}


		public static string getGenre(string UniqueProgrammeIdentifier)
		{
			string Genre = "";

			// create the command object and store the sql query
			OleDbCommand aCommand = new OleDbCommand("select class from Genre where program_id='" + UniqueProgrammeIdentifier + "'", zapConnection);
			OleDbDataReader aReader = aCommand.ExecuteReader();

			//Iterate throuth the database
			while (aReader.Read())
			{
				Genre = aReader.GetString(0);
			}

			//close the reader
			aReader.Close();

			return Genre;
		}

		public static ArrayList getCast(string UniqueProgrammeIdentifier)
		{
			ArrayList list = new ArrayList();
			OleDbCommand aCommand = new OleDbCommand( "select DISTINCT givenname,surname,role FROM Crew where program_id='" + UniqueProgrammeIdentifier + "' ORDER BY role", zapConnection);

			OleDbDataReader aReader = aCommand.ExecuteReader();

			//Iterate throuth the database
			while(aReader.Read())
			{
				Crew crew = new Crew();
				crew.givenname = aReader.GetString(0);
				crew.surname = aReader.GetString(1);
				crew.role = (Role)aReader.GetByte(2);
				list.Add(crew);
			}
			//close the reader
			aReader.Close();
			return list;
		}

		public struct Crew
		{
			public string givenname;
			public string surname;
			public Role role;
		}
	}
}
